"
GsTestSuite is the GemStone-specific refinement of the SUnit TestSuite class.
"
Class {
	#name : 'GsTestSuite',
	#superclass : 'TestSuite',
	#category : nil
}

{ #category : 'Accessing' }
GsTestSuite class >> firstElementOfNRS [

  | firstElement dbfNRS |

  firstElement := SystemRepository fileNames at: 1.
  dbfNRS := (firstElement isKindOf: Array)
    ifTrue:  [ firstElement first ] "32-bit has an array of arrays of { extent, replicate}"
    ifFalse: [ firstElement].       "64-bit has an array of strings"
  ^ dbfNRS

]

{ #category : 'Accessing' }
GsTestSuite class >> logFilePath [

  | path |
  path := './SUnit.log'.
  ^ path .

]

{ #category : 'Debugging' }
GsTestSuite >> _asExecString: runStr [
  | str lf |
  str := String new . lf := Character lf .
  str addAll:'run'; add: lf  .
  self tests do: [:each |
    str addAll: each class name ; addAll: runStr ;
      addAll: ' #'; addAll: each selector ; addAll: ' .'; add: lf
  ].
  str add: $% ; add: lf .
  ^ str

]

{ #category : 'Adding' }
GsTestSuite >> addAllTestsTo: aList [

	tests do: [:each |
		each addAllTestsTo: aList.
	].

]

{ #category : 'Accessing' }
GsTestSuite >> allTests [

	| list |
	list := OrderedCollection new.
	self addAllTestsTo: list.
	^list.

]

{ #category : 'Debugging' }
GsTestSuite >> asDebugString [
^ self _asExecString: ' debug: '

]

{ #category : 'Debugging' }
GsTestSuite >> asRunString [
^ self _asExecString: ' run: '

]

{ #category : 'Accessing' }
GsTestSuite >> errorIfDefects [

        | result list |
        result := self run.
        (list := result defects) notEmpty ifTrue: [list error: 'Found ' , list size printString , ' defects'].
        ^true.

]

{ #category : 'Running' }
GsTestSuite >> run [
  | result |
  result := GsTestResult new.
  GsTestCase openSunitLog: #filesystem .
  [self run: result] sunitEnsure: [self resources do: [:each | each reset]].
  GsTestCase flushLog .
  ^result

]

{ #category : 'Running' }
GsTestSuite >> run: aResult [

  self tests do: [:each |  "each may be either a TestSuite or a TestCase"
    self sunitChanged: each.
    each run: aResult
  ]

]
